#ifdef WIN32
/* fix a visualstudio 2005 bug */
#include <omp.h>
#endif
#include <iostream>
#include <iomanip>
#include <math.h>
#include "realtime.h"
using namespace std;

extern double CalcPi (int n, int iRank, int iNumProc);

int main(int argc, char **argv)
{
    int n = 150000000;
    int iMyRank, iNumProcs;
    const double fPi25DT = 3.141592653589793238462643;

    iMyRank = 0;
    iNumProcs = 1; 

#ifdef READ_INPUT  
    if (iMyRank == 0) 
    {
        cout << "Enter the number of intervals: ";
        cin >> n;
    }
#endif

    double fTimeStart = GetRealTime();
    
    if (n <= 0 || n > 2147483647)
    {
       cout << "\n given value has to be between 0 and 2147483647 ..\n";
       return 1;
    }

    /* the calculation is done here*/
    double fPi = CalcPi(n, iMyRank, iNumProcs);


    double fTimeEnd = GetRealTime();

    if (iMyRank == 0)
    {
        cout << "\npi is approximately = " << setprecision(20) << fPi << "\n"
             << "Error is            = " << fabs(fPi - fPi25DT) << "\n"
             << "wall clock time     = " << fTimeEnd - fTimeStart << "\n";
    }
    return 0;
}

double f(double a)
{
    return (4.0 / (1.0 + a*a));
}

double CalcPi (int n, int iRank, int iNumProcs)
{
    const double fH   = 1.0 / (double) n;
    double fSum = 0.0;

#pragma omp parallel for reduction(+:fSum)
    for (int i = iRank; i < n; i += iNumProcs)
    {
        double fX = fH * ((double)i + 0.5);
        fSum += f(fX);
    }  
    return (fH * fSum);
}
