#include <stdio.h>
#include <math.h>
#include <stdlib.h>
#ifdef WIN32
/* fix a visualstudio 2005 bug */
#include <omp.h>
#endif
#include "realtime.h"

double CalcPi (int n, int iRank, int iNumProcs);

int main(int argc, char **argv)
{
    int n = 150000000;
    int iMyRank, iNumProcs;
    const double fPi25DT = 3.141592653589793238462643;
    double fPi;
    double fTimeStart, fTimeEnd;

    iMyRank = 0;
    iNumProcs = 1;
    
#ifdef READ_INPUT  
    if (iMyRank == 0) 
    {
        printf("Enter the number of intervals: ");
        scanf("%d",&n);
    }
#endif

    fTimeStart = GetRealTime();
    
    if (n <= 0 || n > 2147483647 ) 
    {
        printf("\ngiven value has to be between 0 and 2147483647\n");
        return 1;
    }

    /* the calculation is done here*/
    fPi = CalcPi(n, iMyRank, iNumProcs);


    fTimeEnd = GetRealTime();

    if (iMyRank == 0)
    {
        printf("\npi is approximately = %.20f \nError               = %.20f\n",
               fPi, fabs(fPi - fPi25DT));
        printf(  "wall clock time     = %.20f\n", fTimeEnd - fTimeStart);
    }
    return 0;
}


double f(double a)
{
    return (4.0 / (1.0 + a*a));
}

double CalcPi (int n, int iRank, int iNumProcs)
{
    const double fH   = 1.0 / (double) n;
    double fSum = 0.0;
    double fX;
    int i;

#pragma omp parallel for private(i, fX) reduction(+:fSum)
    for (i = iRank; i < n; i += iNumProcs)
    {
        fX = fH * ((double)i + 0.5);
        fSum += f(fX);
    }
    return fH * fSum;
}
