include "realtime.f90"
Program Pi
    use RealTime
    implicit none
    integer(kind=8) :: n
    integer :: iMyRank, iNumProcs, iErr
    double precision, parameter :: fPi25DT = 3.141592653589793238462643d0
    double precision :: fLocalPi, fPi
    double precision :: fTimeStart, fTimeEnd, CalcPi

    n = 150000000

    iMyRank = 0
    iNumProcs = 1

#ifdef READ_INPUT  
    if (iMyRank == 0) then
        write (*,*) 'Enter the number of intervals: '
        read (*,*) n
    end if
#endif

    fTimeStart = GetRealTime()

    if (n <= 0 .and. n > 2147483647) then
        write (*,*) "given value has to be between 0 and 2147483647 ..\n"
        stop
    end if

!    /* the calculation is done here*/
    fPi = CalcPi(n, iMyRank, iNumProcs)


    fTimeEnd = GetRealTime()

    if (iMyRank == 0) then
        write (*,327) 'pi is approximately = ', fPi
        write (*,327) 'Error is            = ', abs(fPi - fPi25DT)
        write (*,327) 'wall clock time     = ', fTimeEnd - fTimeStart
327     format (A22, F22.20)
    end if
end program


double precision function f(a)
    double precision, intent(in) :: a
  
    f = 4.0 / (1.0 + a*a)
    return
end function


double precision function CalcPi (n, iRank, iNumProcs)
    implicit none
    integer(kind=8), intent(in) :: n
    integer, intent(in) :: iRank, iNumProcs
    double precision :: fh, fSum, fX, f
    integer(kind=8) :: i
    
    fH   = 1.0 / dble(n)
    fSum = 0.0

    do i = iRank, n -1, iNumProcs
        fX = fH * (dble(i) + 0.5)
        fSum = fSum + f(fX)
    end do

    CalcPi = fH * fSum
    return
end function

